
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity ioBus is
    Port ( 
		ClkxCI 		: in  STD_LOGIC;
		RstxRI		: in  std_logic;
		-- master interface from cpu
		CpuCycxSI 	: IN  std_logic;
        CpuStbxSI 	: IN  std_logic;
        CpuWexSI  	: IN  std_logic;
        CpuAdrxDI 	: IN  std_logic_vector(5 downto 0);
        CpuDatxDI 	: IN  std_logic_vector(7 downto 0);
        CpuDatxDO 	: OUT  std_logic_vector(7 downto 0);
        CpuAckxSO 	: out  std_logic;
        CpuErrxSO 	: out  std_logic;    
		CpuRtyxSO 	: out  std_logic;
		-- interrupts
		UartIntxSO	: out  std_logic;
		SpiIntxSO	: out  std_logic;
		-- uart io pins
		UartTxxDO	: out std_logic;
		UartRxxDI	: in  std_logic;
		UartRtsxSO	: out std_logic;
		UartCtsxSI	: in  std_logic;
		-- spi pins
		SpiSckxSO	: out std_logic;
		SpiMisoxDI	: in  std_logic;
		SpiMosixDO	: out std_logic;
		SpiSSxSO	: out std_logic_vector(2 downto 0);
		-- gpio pins                      
		SwitchxDI	: in  std_logic_vector(3 downto 0);
		ButtonxDI	: in  std_logic_vector(3 downto 0);
		LedxDO		: out std_logic_vector(7 downto 0)		
	);
end ioBus;

architecture Behavioral of ioBus is

	-- wishbone bus connector
	component wb_conbus_top is
	Generic (
		dw 				: integer;
		aw				: integer;
		s0_addr_w 		: integer;
		s0_addr			: integer;
		s1_addr_w		: integer;
		s1_addr			: integer;
		s27_addr_w		: integer;
		s2_addr 		: integer;
		s3_addr 		: integer;
		s4_addr 		: integer;
		s5_addr 		: integer;
		s6_addr 		: integer;
		s7_addr 		: integer);
	Port (
		clk_i		: in  std_logic;
		rst_i		: in  std_logic;
		-- Master 0 Interface
		m0_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m0_dat_o	: out std_logic_vector(dw-1 downto 0);
		m0_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m0_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m0_we_i		: in  std_logic;
		m0_cyc_i	: in  std_logic;
		m0_stb_i	: in  std_logic;
		m0_ack_o	: out std_logic;
		m0_err_o	: out std_logic;
		m0_rty_o	: out std_logic;
		m0_cab_i	: in  std_logic;
		
		-- Master 1 Interface
		m1_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m1_dat_o	: out std_logic_vector(dw-1 downto 0);
		m1_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m1_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m1_we_i		: in  std_logic;
		m1_cyc_i	: in  std_logic;
		m1_stb_i	: in  std_logic;
		m1_ack_o	: out std_logic;
		m1_err_o	: out std_logic;
		m1_rty_o	: out std_logic;
		m1_cab_i	: in  std_logic;

		-- Master 2 Interface
		m2_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m2_dat_o	: out std_logic_vector(dw-1 downto 0);
		m2_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m2_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m2_we_i		: in  std_logic;
		m2_cyc_i	: in  std_logic;
		m2_stb_i	: in  std_logic;
		m2_ack_o	: out std_logic;
		m2_err_o	: out std_logic;
		m2_rty_o	: out std_logic;
		m2_cab_i	: in  std_logic;
		
		-- Master 3 Interface
		m3_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m3_dat_o	: out std_logic_vector(dw-1 downto 0);
		m3_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m3_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m3_we_i		: in  std_logic;
		m3_cyc_i	: in  std_logic;
		m3_stb_i	: in  std_logic;
		m3_ack_o	: out std_logic;
		m3_err_o	: out std_logic;
		m3_rty_o	: out std_logic;
		m3_cab_i	: in  std_logic;
		
		-- Master 4 Interface
		m4_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m4_dat_o	: out std_logic_vector(dw-1 downto 0);
		m4_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m4_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m4_we_i		: in  std_logic;
		m4_cyc_i	: in  std_logic;
		m4_stb_i	: in  std_logic;
		m4_ack_o	: out std_logic;
		m4_err_o	: out std_logic;
		m4_rty_o	: out std_logic;
		m4_cab_i	: in  std_logic;
		
		-- Master 5 Interface
		m5_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m5_dat_o	: out std_logic_vector(dw-1 downto 0);
		m5_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m5_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m5_we_i		: in  std_logic;
		m5_cyc_i	: in  std_logic;
		m5_stb_i	: in  std_logic;
		m5_ack_o	: out std_logic;
		m5_err_o	: out std_logic;
		m5_rty_o	: out std_logic;
		m5_cab_i	: in  std_logic;
		
		-- Master 6 Interface
		m6_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m6_dat_o	: out std_logic_vector(dw-1 downto 0);
		m6_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m6_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m6_we_i		: in  std_logic;
		m6_cyc_i	: in  std_logic;
		m6_stb_i	: in  std_logic;
		m6_ack_o	: out std_logic;
		m6_err_o	: out std_logic;
		m6_rty_o	: out std_logic;
		m6_cab_i	: in  std_logic;
		
		-- Master 7 Interface
		m7_dat_i	: in  std_logic_vector(dw-1 downto 0);
		m7_dat_o	: out std_logic_vector(dw-1 downto 0);
		m7_adr_i	: in  std_logic_vector(aw-1 downto 0);
		m7_sel_i	: in  std_logic_vector((dw/8)-1 downto 0);
		m7_we_i		: in  std_logic;
		m7_cyc_i	: in  std_logic;
		m7_stb_i	: in  std_logic;
		m7_ack_o	: out std_logic;
		m7_err_o	: out std_logic;
		m7_rty_o	: out std_logic;
		m7_cab_i	: in  std_logic;

		--	// Slave 0 Interface
		s0_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s0_dat_o	: out std_logic_vector(dw-1 downto 0);
		s0_adr_o	: out std_logic_vector(aw-1 downto 0);
		s0_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s0_we_o		: out std_logic;
		s0_cyc_o	: out std_logic;
		s0_stb_o	: out std_logic;
		s0_ack_i	: in  std_logic;
		s0_err_i	: in  std_logic;
		s0_rty_i	: in  std_logic;
		s0_cab_o	: out std_logic;
		
		--	// Slave 1 Interface
		s1_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s1_dat_o	: out std_logic_vector(dw-1 downto 0);
		s1_adr_o	: out std_logic_vector(aw-1 downto 0);
		s1_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s1_we_o		: out std_logic;
		s1_cyc_o	: out std_logic;
		s1_stb_o	: out std_logic;
		s1_ack_i	: in  std_logic;
		s1_err_i	: in  std_logic;
		s1_rty_i	: in  std_logic;
		s1_cab_o	: out std_logic;
		
		--	// Slave 2 Interface
		s2_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s2_dat_o	: out std_logic_vector(dw-1 downto 0);
		s2_adr_o	: out std_logic_vector(aw-1 downto 0);
		s2_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s2_we_o		: out std_logic;
		s2_cyc_o	: out std_logic;
		s2_stb_o	: out std_logic;
		s2_ack_i	: in  std_logic;
		s2_err_i	: in  std_logic;
		s2_rty_i	: in  std_logic;
		s2_cab_o	: out std_logic;
		
		--	// Slave 3 Interface
		s3_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s3_dat_o	: out std_logic_vector(dw-1 downto 0);
		s3_adr_o	: out std_logic_vector(aw-1 downto 0);
		s3_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s3_we_o		: out std_logic;
		s3_cyc_o	: out std_logic;
		s3_stb_o	: out std_logic;
		s3_ack_i	: in  std_logic;
		s3_err_i	: in  std_logic;
		s3_rty_i	: in  std_logic;
		s3_cab_o	: out std_logic;
		
		--	// Slave 4 Interface
		s4_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s4_dat_o	: out std_logic_vector(dw-1 downto 0);
		s4_adr_o	: out std_logic_vector(aw-1 downto 0);
		s4_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s4_we_o		: out std_logic;
		s4_cyc_o	: out std_logic;
		s4_stb_o	: out std_logic;
		s4_ack_i	: in  std_logic;
		s4_err_i	: in  std_logic;
		s4_rty_i	: in  std_logic;
		s4_cab_o	: out std_logic;
		
		--	// Slave 5 Interface
		s5_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s5_dat_o	: out std_logic_vector(dw-1 downto 0);
		s5_adr_o	: out std_logic_vector(aw-1 downto 0);
		s5_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s5_we_o		: out std_logic;
		s5_cyc_o	: out std_logic;
		s5_stb_o	: out std_logic;
		s5_ack_i	: in  std_logic;
		s5_err_i	: in  std_logic;
		s5_rty_i	: in  std_logic;
		s5_cab_o	: out std_logic;
		
		--	// Slave 6 Interface
		s6_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s6_dat_o	: out std_logic_vector(dw-1 downto 0);
		s6_adr_o	: out std_logic_vector(aw-1 downto 0);
		s6_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s6_we_o		: out std_logic;
		s6_cyc_o	: out std_logic;
		s6_stb_o	: out std_logic;
		s6_ack_i	: in  std_logic;
		s6_err_i	: in  std_logic;
		s6_rty_i	: in  std_logic;
		s6_cab_o	: out std_logic;
		
		--	// Slave 7 Interface
		s7_dat_i	: in  std_logic_vector(dw-1 downto 0);
		s7_dat_o	: out std_logic_vector(dw-1 downto 0);
		s7_adr_o	: out std_logic_vector(aw-1 downto 0);
		s7_sel_o	: out std_logic_vector((dw/8)-1 downto 0);
		s7_we_o		: out std_logic;
		s7_cyc_o	: out std_logic;
		s7_stb_o	: out std_logic;
		s7_ack_i	: in  std_logic;
		s7_err_i	: in  std_logic;
		s7_rty_i	: in  std_logic;
		s7_cab_o	: out std_logic
	);
	end component;
	
	component uart_top is
	port (
		wb_clk_i		: in  std_logic;
		-- Wishbone signals
		wb_rst_i		: in  std_logic;
		wb_adr_i		: in  std_logic_vector(2 downto 0);
		wb_dat_i		: in  std_logic_vector(7 downto 0);
		wb_dat_o		: out std_logic_vector(7 downto 0);
		wb_we_i			: in  std_logic;
		wb_stb_i		: in  std_logic;
		wb_cyc_i		: in  std_logic;
		wb_ack_o		: out std_logic;
		wb_sel_i		: in  std_logic_vector(3 downto 0);
		int_o			: out std_logic;  -- interrupt request
		-- UART	signals
		stx_pad_o		: out std_logic;
		srx_pad_i		: in  std_logic;
		-- modem signals
		rts_pad_o		: out std_logic;
		cts_pad_i		: in  std_logic;
		dtr_pad_o		: out std_logic;
		dsr_pad_i		: in  std_logic;
		ri_pad_i		: in  std_logic;
		dcd_pad_i		: in  std_logic);
	end component;

	component simple_spi_top is
	port (
		-- 8 Bit Wishbone interface
		clk_i			: in  std_logic;
		rst_i			: in  std_logic;
		cyc_i			: in  std_logic;
		stb_i			: in  std_logic;
		adr_i			: in  std_logic_vector(1 downto 0);
		we_i			: in  std_logic;
		dat_i			: in  std_logic_vector(7 downto 0);
		dat_o			: out std_logic_vector(7 downto 0);
		ack_o			: out std_logic;
		inta_o			: out std_logic;
		-- SPI port
		sck_o			: out std_logic;
		mosi_o			: out std_logic;
		miso_i			: in  std_logic);
	end component;

	signal RstxRB				: std_logic;
	signal LedxDN, LedxDP		: std_logic_vector(7 downto 0);
	signal SSxDN, SSxDP			: std_logic_vector(2 downto 0);
	signal InputsxDN, InputsxDP	: std_logic_vector(7 downto 0);
	
	-- wb bus for uart
	signal UartCycxS			: std_logic;
	signal UartStbxS			: std_logic;
	signal UartWexS				: std_logic;
	signal UartAdrxD			: std_logic_vector(5 downto 0);
	signal UartDatMISOxD		: std_logic_vector(7 downto 0);
	signal UartDatMOSIxD		: std_logic_vector(7 downto 0);
	signal UartAckxS			: std_logic;
	
	-- dummy bus on slave 1
	signal DummyCycxS			: std_logic;
	signal DummyStbxS			: std_logic;
	signal DummyAckxSN			: std_logic;
	signal DummyAckxSP			: std_logic;
	
	-- wb bus for gpio
	signal GpioCycxS			: std_logic;
	signal GpioStbxS			: std_logic;
	signal GpioWexS				: std_logic;
	signal GpioAdrxD			: std_logic_vector(5 downto 0);
	signal GpioDatMISOxD		: std_logic_vector(7 downto 0);
	signal GpioDatMOSIxD		: std_logic_vector(7 downto 0);
	signal GpioAckxS			: std_logic;
	
	-- wb bus for spi
	signal SpiCycxS				: std_logic;
	signal SpiStbxS				: std_logic;
	signal SpiWexS				: std_logic;
	signal SpiAdrxD				: std_logic_vector(5 downto 0);
	signal SpiDatMISOxD			: std_logic_vector(7 downto 0);
	signal SpiDatMOSIxD			: std_logic_vector(7 downto 0);
	signal SpiAckxS				: std_logic;
	

begin

	WbIoBus : wb_conbus_top
	Generic Map (
		dw 				=> 8,
		aw				=> 6,
		s0_addr_w 		=> 3,	-- how many bits are decoded to address the uart
		s0_addr			=> 6,	-- decoded (higher) bits of uart address
		s1_addr_w		=> 3,	-- dummy bus access acknowledger
		s1_addr			=> 7,
		s27_addr_w		=> 4,
		s2_addr 		=> 0,	-- gpio
		s3_addr 		=> 1,   -- spi
		s4_addr 		=> 2,	-- all others: unused
		s5_addr 		=> 3,
		s6_addr 		=> 4,
		s7_addr 		=> 5)
	Port Map (
		clk_i		=> ClkxCI,
		rst_i		=> RstxRI,
		-- Master 0 Interface
		m0_dat_i	=> CpuDatxDI,
		m0_dat_o	=> CpuDatxDO,
		m0_adr_i	=> CpuAdrxDI,
		m0_sel_i	=> "1",		-- dummy signal, no use in 8 bit bus
		m0_we_i		=> CpuWexSI,
		m0_cyc_i	=> CpuCycxSI,
		m0_stb_i	=> CpuStbxSI,
		m0_ack_o	=> CpuAckxSO,
		m0_err_o	=> CpuErrxSO,
		m0_rty_o	=> CpuRtyxSO,
		m0_cab_i	=> '0',
		-- Master 1 Interface
		m1_dat_i	=> x"00",
		m1_dat_o	=> open,
		m1_adr_i	=> "000000",
		m1_sel_i	=> "0",
		m1_we_i		=> '0',
		m1_cyc_i	=> '0',
		m1_stb_i	=> '0',
		m1_ack_o	=> open,
		m1_err_o	=> open,
		m1_rty_o	=> open,  
		m1_cab_i	=> '0', 
		-- Master 2 Interface
		m2_dat_i	=> x"00",
		m2_dat_o	=> open,
		m2_adr_i	=> "000000",
		m2_sel_i	=> "0",
		m2_we_i		=> '0',
		m2_cyc_i	=> '0',
		m2_stb_i	=> '0',
		m2_ack_o	=> open,
		m2_err_o	=> open,
		m2_rty_o	=> open,  
		m2_cab_i	=> '0', 
		-- Master 3 Interface
		m3_dat_i	=> x"00",
		m3_dat_o	=> open,
		m3_adr_i	=> "000000",
		m3_sel_i	=> "0",
		m3_we_i		=> '0',
		m3_cyc_i	=> '0',
		m3_stb_i	=> '0',
		m3_ack_o	=> open,
		m3_err_o	=> open,
		m3_rty_o	=> open,  
		m3_cab_i	=> '0', 
		-- Master 4 Interface
		m4_dat_i	=> x"00",
		m4_dat_o	=> open,
		m4_adr_i	=> "000000",
		m4_sel_i	=> "0",
		m4_we_i		=> '0',
		m4_cyc_i	=> '0',
		m4_stb_i	=> '0',
		m4_ack_o	=> open,
		m4_err_o	=> open,
		m4_rty_o	=> open,  
		m4_cab_i	=> '0', 
		-- Master 5 Interface
		m5_dat_i	=> x"00",
		m5_dat_o	=> open,
		m5_adr_i	=> "000000",
		m5_sel_i	=> "0",
		m5_we_i		=> '0',
		m5_cyc_i	=> '0',
		m5_stb_i	=> '0',
		m5_ack_o	=> open,
		m5_err_o	=> open,
		m5_rty_o	=> open,  
		m5_cab_i	=> '0', 
		-- Master 6 Interface
		m6_dat_i	=> x"00",
		m6_dat_o	=> open,
		m6_adr_i	=> "000000",
		m6_sel_i	=> "0",
		m6_we_i		=> '0',
		m6_cyc_i	=> '0',
		m6_stb_i	=> '0',
		m6_ack_o	=> open,
		m6_err_o	=> open,
		m6_rty_o	=> open,  
		m6_cab_i	=> '0', 
		-- Master 7 Interface
		m7_dat_i	=> x"00",
		m7_dat_o	=> open,
		m7_adr_i	=> "000000",
		m7_sel_i	=> "0",
		m7_we_i		=> '0',
		m7_cyc_i	=> '0',
		m7_stb_i	=> '0',
		m7_ack_o	=> open,
		m7_err_o	=> open,
		m7_rty_o	=> open,  
		m7_cab_i	=> '0', 
		--	Slave 0 Interface - UART       
		s0_dat_i	=> UartDatMISOxD,
		s0_dat_o	=> UartDatMOSIxD,
		s0_adr_o	=> UartAdrxD,
		s0_sel_o	=> open,
		s0_we_o		=> UartWexS,
		s0_cyc_o	=> UartCycxS,
		s0_stb_o	=> UartStbxS,
		s0_ack_i	=> UartAckxS,
		s0_err_i	=> '0',
		s0_rty_i	=> '0',
		s0_cab_o	=> open,            
		--	Slave 1 Interface - dummy ack generator, compiler access this address space for no good reason
		s1_dat_i	=> x"00",
		s1_dat_o	=> open,
		s1_adr_o	=> open,
		s1_sel_o	=> open,
		s1_we_o		=> open,
		s1_cyc_o	=> DummyCycxS,
		s1_stb_o	=> DummyStbxS,
		s1_ack_i	=> DummyAckxSP,
		s1_err_i	=> '0',
		s1_rty_i	=> '0',
		s1_cab_o	=> open,
		--	Slave 2 Interface - GPIO
		s2_dat_i	=> GpioDatMISOxD,
		s2_dat_o	=> GpioDatMOSIxD,
		s2_adr_o	=> GpioAdrxD,
		s2_sel_o	=> open,
		s2_we_o		=> GpioWexS,
		s2_cyc_o	=> GpioCycxS,
		s2_stb_o	=> GpioStbxS,
		s2_ack_i	=> GpioAckxS,
		s2_err_i	=> '0',
		s2_rty_i	=> '0',
		s2_cab_o	=> open,
		--	Slave 3 Interface - SPI
		s3_dat_i	=> SpiDatMISOxD,
		s3_dat_o	=> SpiDatMOSIxD,
		s3_adr_o	=> SpiAdrxD,
		s3_sel_o	=> open,
		s3_we_o		=> SpiWexS,
		s3_cyc_o	=> SpiCycxS,
		s3_stb_o	=> SpiStbxS,
		s3_ack_i	=> SpiAckxS,
		s3_err_i	=> '0',
		s3_rty_i	=> '0',
		s3_cab_o	=> open,
		--	Slave 4 Interface - unused
		s4_dat_i	=> x"00",
		s4_dat_o	=> open,
		s4_adr_o	=> open,
		s4_sel_o	=> open,
		s4_we_o		=> open,
		s4_cyc_o	=> open,
		s4_stb_o	=> open,
		s4_ack_i	=> '0',
		s4_err_i	=> '0',
		s4_rty_i	=> '0',
		s4_cab_o	=> open,
		--	Slave 5 Interface - unused
		s5_dat_i	=> x"00",
		s5_dat_o	=> open,
		s5_adr_o	=> open,
		s5_sel_o	=> open,
		s5_we_o		=> open,
		s5_cyc_o	=> open,
		s5_stb_o	=> open,
		s5_ack_i	=> '0',
		s5_err_i	=> '0',
		s5_rty_i	=> '0',
		s5_cab_o	=> open,
		--	Slave 6 Interface - unused
		s6_dat_i	=> x"00",
		s6_dat_o	=> open,
		s6_adr_o	=> open,
		s6_sel_o	=> open,
		s6_we_o		=> open,
		s6_cyc_o	=> open,
		s6_stb_o	=> open,
		s6_ack_i	=> '0',
		s6_err_i	=> '0',
		s6_rty_i	=> '0',
		s6_cab_o	=> open,
		--	Slave 7 Interface - unused
		s7_dat_i	=> x"00",
		s7_dat_o	=> open,
		s7_adr_o	=> open,
		s7_sel_o	=> open,
		s7_we_o		=> open,
		s7_cyc_o	=> open,
		s7_stb_o	=> open,
		s7_ack_i	=> '0',
		s7_err_i	=> '0',
		s7_rty_i	=> '0',
		s7_cab_o	=> open	
	);

	RstxRB	<= not RstxRI;

	-- GPIO
	
	InputsxDN <= SwitchxDI & ButtonxDI;	-- synchronize the button- and switch- inputs to the clock signal
	
	process (GpioCycxS, GpioStbxS, GpioWexS, GpioAdrxD, GpioDatMOSIxD, LedxDP, SSxDP, InputsxDP)
	begin
		LedxDN	<= LedxDP;
		SSxDN	<= SSxDP;
		-- update the register on wirte accesses
		if GpioCycxS='1' and GpioStbxS='1' and GpioWexS='1' then
			if GpioAdrxD(1 downto 0)="00" then
				LedxDN 	<= GpioDatMOSIxD;
			end if;
			if GpioAdrxD(1 downto 0)="01" then
				SSxDN	<= GpioDatMOSIxD(2 downto 0);
			end if;	
		end if;
		-- decode address to find the right data input for the cpu bus
		case GpioAdrxD(1 downto 0) is
			 when "00" => GpioDatMISOxD <= LedxDP;
			 when "01" => GpioDatMISOxD <= "11111" & SSxDP;
			 when "10" => GpioDatMISOxD <= InputsxDP;
			 when others => GpioDatMISOxD <= x"00";
		end case;
	end process;
	
	
	
	process (RstxRI, ClkxCI)
	begin
		if RstxRI='1' then
			LedxDP		<= (others => '0');
			SSxDP		<= (others => '1');
			InputsxDP	<= (others => '0');
		elsif ClkxCI'event and ClkxCI='1' then
			LedxDP		<= LedxDN;
			SSxDP		<= SSxDN;
			InputsxDP	<= InputsxDN;
			
			GpioAckxS	<= GpioCycxS and GpioStbxS and (not GpioAckxS);
			
		end if;         
	end process;
	
             
	SpiSSxSO	<= SSxDP;
	LedxDO		<= LedxDP;

	-- dummy bus ack genreator
	DummyAckxSN	<= DummyCycxS and DummyStbxS and (not DummyAckxSP);
	process (ClkxCI)
	begin
		if ClkxCI'event and ClkxCI='1' then
			DummyAckxSP	<= DummyAckxSN;
		end if;
	end process;

	-- uart
	UartInst : uart_top 
	port map (
		wb_clk_i	=> ClkxCI,
		-- Wishbone signals
		wb_rst_i	=> RstxRI,
		wb_adr_i	=> UartAdrxD(2 downto 0),
		wb_dat_i	=> UartDatMOSIxD,
		wb_dat_o	=> UartDatMISOxD,
		wb_we_i		=> UartWexS,
		wb_stb_i	=> UartStbxS,
		wb_cyc_i	=> UartCycxS,
		wb_ack_o	=> UartAckxS,
		wb_sel_i	=> "1111",	-- this should not be here, sel is only used for 32 bit busses, this is a bug in the uart
		int_o		=> UartIntxSO,
		-- UART	signals                               
		stx_pad_o	=> UartTxxDO,
		srx_pad_i	=> UartRxxDI,
		-- modem signals
		rts_pad_o	=> UartRtsxSO,
		cts_pad_i	=> UartCtsxSI,
		dtr_pad_o	=> open,
		dsr_pad_i	=> '0',
		ri_pad_i	=> '0',
		dcd_pad_i	=> '0'
	);
	
	-- SPI
	SpiInst : simple_spi_top 
	port map (
		-- 8 Bit Wishbone interface
		clk_i		=> ClkxCI,
		rst_i		=> RstxRB,	-- active low!
		cyc_i		=> SpiCycxS,
		stb_i		=> SpiStbxS,
		adr_i		=> SpiAdrxD(1 downto 0),
		we_i		=> SpiWexS,
		dat_i		=> SpiDatMOSIxD,
		dat_o		=> SpiDatMISOxD,
		ack_o		=> SpiAckxS,
		inta_o		=> SpiIntxSO,
		-- SPI port
		sck_o		=> SpiSckxSO,
		mosi_o		=> SpiMosixDO,
		miso_i		=> SpiMisoxDI
	);
	

end Behavioral;

