
LIBRARY ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_arith.ALL;
 

ENTITY Wb_sump_interface_tb IS
END Wb_sump_interface_tb;
 
ARCHITECTURE behavior OF Wb_sump_interface_tb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT Wb_sump_interface
    PORT(
         RstxRI : IN  std_logic;
         ClkxCI : IN  std_logic;
         Wb_cycxSI : IN  std_logic;
         Wb_stbxSI : IN  std_logic;
         Wb_wexSI : IN  std_logic;
         Wb_ackxSO : OUT  std_logic;
         Wb_errxSO : OUT  std_logic;
         Wb_adrxDI : IN  std_logic_vector(3 downto 0);
         Wb_datxDI : IN  std_logic_vector(7 downto 0);
		 Wb_datxDO	: OUT std_logic_vector(7 downto 0);
         Clk_sumpxCI : IN  std_logic;
		 channelsxDI : in  std_logic_vector(31 downto 0);
         cmd_dataxDO : OUT  std_logic_vector(31 downto 0);
         opcodexDO : OUT  std_logic_vector(7 downto 0);
         executexSO : OUT  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal RstxRI : std_logic := '1';
   signal ClkxCI : std_logic := '0';
   signal Wb_cycxSI : std_logic := '0';
   signal Wb_stbxSI : std_logic := '0';
   signal Wb_wexSI : std_logic := '0';
   signal Wb_adrxDI : std_logic_vector(3 downto 0) := (others => '0');
   signal Wb_datxDI : std_logic_vector(7 downto 0) := (others => '0');
   signal Clk_sumpxCI : std_logic := '0';
   signal channelsxDI : std_logic_vector(31 downto 0) := (others => '0');

 	--Outputs
   signal Wb_ackxSO : std_logic;
   signal Wb_errxSO : std_logic;
   signal Wb_datxDO : std_logic_vector(7 downto 0);
   signal cmd_dataxDO : std_logic_vector(31 downto 0);
   signal opcodexDO : std_logic_vector(7 downto 0);
   signal executexSO : std_logic;

   -- Clock period definitions
   constant ClkxCI_period : time := 25 ns; --40MHz AVR clock
   constant Clk_sumpxCI_period : time := 10 ns; -- 100MHz sump clock
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: Wb_sump_interface PORT MAP (
          RstxRI => RstxRI,
          ClkxCI => ClkxCI,
          Wb_cycxSI => Wb_cycxSI,
          Wb_stbxSI => Wb_stbxSI,
          Wb_wexSI => Wb_wexSI,
          Wb_ackxSO => Wb_ackxSO,
          Wb_errxSO => Wb_errxSO,
          Wb_adrxDI => Wb_adrxDI,
          Wb_datxDI => Wb_datxDI,
		  Wb_datxDO => Wb_datxDO,
          Clk_sumpxCI => Clk_sumpxCI,
		  channelsxDI => channelsxDI,
          cmd_dataxDO => cmd_dataxDO,
          opcodexDO => opcodexDO,
          executexSO => executexSO
        );

   -- Clock process definitions
   ClkxCI_process : process
   begin
		ClkxCI <= '0';
		wait for ClkxCI_period/2;
		ClkxCI <= '1';
		wait for ClkxCI_period/2;
   end process;
 
   Clk_sumpxCI_process : process
   begin
		Clk_sumpxCI <= '0';
		wait for Clk_sumpxCI_period/2;
		Clk_sumpxCI <= '1';
		wait for Clk_sumpxCI_period/2;
   end process;
 
   -- generate a counter on the channel inputs
   channels_stim_process : process (Clk_sumpxCI)
   begin
      if Clk_sumpxCI'event and Clk_sumpxCI='1' then
	     channelsxDI <= std_logic_vector(conv_unsigned(conv_integer(unsigned(channelsxDI))+1,32));
	  end if;
   end process;

   -- Stimulus process
   stim_proc : process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns;	
		RstxRI <= '0';
		
      wait for ClkxCI_period*10;

      --generate 4 write-accesses:
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0000";
		Wb_datxDI <= x"05";--data:5
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0001";
		Wb_datxDI <= x"0A";--data:10
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0010";
		Wb_datxDI <= x"0F";--data:15
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';

		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0011";
		Wb_datxDI <= x"14";--data:20
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		--generate the opcode:
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0100";
		Wb_datxDI <= x"CC";--data:0b11001100
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI <= '0';
		
		--next 4 Bytes for testing:
		wait for ClkxCI_period*10;

        --generate 4 write-accesses:
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0000";
		Wb_datxDI <= x"0A";
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0001";
		Wb_datxDI <= x"0B";
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0010";
		Wb_datxDI <= x"0C";
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';

		wait for 100ns;
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0011";
		Wb_datxDI <= x"0D";
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for 100ns;
		
		--generate the opcode:
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"0100";
		Wb_datxDI <= x"AB";
		Wb_wexSI <= '1';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI <= '0';
		
		-- read current sampling channel state
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"1000";
		Wb_datxDI <= x"00";
		Wb_wexSI  <= '0';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI  <= '0';
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"1001";
		Wb_datxDI <= x"00";
		Wb_wexSI  <= '0';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI  <= '0';
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"1010";
		Wb_datxDI <= x"00";
		Wb_wexSI  <= '0';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI  <= '0';
		
		wait until rising_edge(ClkxCI);
		Wb_adrxDI <= b"1011";
		Wb_datxDI <= x"00";
		Wb_wexSI  <= '0';
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		wait until rising_edge(ClkxCI) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		Wb_wexSI  <= '0';
		
      wait;
   end process;

END;
