/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Header file for low level io like gpio, spi, analyzer core.
--*
--***************************************************************************************************************/

#ifndef __IO_H__
#define __IO_H__


#include <avr/sfr_defs.h>


/**************************************************************************************
*   D E F I N E S   ( H A R D W A R E   P R O P E R T I E S )                        */

/* general purpose io modul*/
#define GPIO_BASE	0x00
/* 8 leds, only four are visible on SP601 fpga board */
#define LEDS		_SFR_IO8(GPIO_BASE+0)
/* the three spi slave select channels of the spi header */
#define SLAVE_SEL	_SFR_IO8(GPIO_BASE+1)
/* 4 dip switches and 4 push-button inputs */
#define BSWIN		_SFR_IO8(GPIO_BASE+2)

/* spi core, see documentation in fpga/spi */
#define SPI_BASE	0x04

#ifdef SPCR
	#undef SPCR
#endif
#ifdef SPSR
	#undef SPSR
#endif
#ifdef SPDR
	#undef SPDR
#endif

#define SPCR		_SFR_IO8(SPI_BASE+0)
#define SPSR		_SFR_IO8(SPI_BASE+1)
#define SPDR		_SFR_IO8(SPI_BASE+2)
#define SPER		_SFR_IO8(SPI_BASE+3)
/* bit locations */
#define SPIE		7
#define SPE			6
#define MSTR		4
#define CPOL		3
#define CPHA		2
#define SPR1		1
#define SPR0		0

#define SPIF		7
#define WCOL		6
#define WFFULL		3
#define WFEMPTY		2
#define RFFULL		1
#define RFEMPTY		0

#define ICNT1		7
#define ICNT0		6
#define ESPR1		1
#define ESPR0		0

/* slave select macros for SPI */
#define SPI_SEL_CH_LEDS()	(SLAVE_SEL = (~(1<<0)))
#define SPI_SEL_1()			(SLAVE_SEL = (~(1<<1)))
#define SPI_SEL_LCD()		(SLAVE_SEL &= (~(1<<2)))
#define SPI_DESELECT()		(SLAVE_SEL = 0xff)

/* la hardware interface */
#define LA_BASE			0x4000
#define LA_DATA_REG		(*((volatile uint32_t*)(LA_BASE)))
#define LA_OPCODE_REG	(*((volatile uint8_t*)(LA_BASE+4)))
#define LA_CHANNELS		(*((volatile uint32_t*)(LA_BASE+8)))


/**************************************************************************************
*   P R O T O T Y P E S                                                              */

void configSpi (uint8_t cpol, uint8_t cpha, uint8_t div);

uint8_t spiTransferByte (uint8_t d);

void initLCD();

void putLCD(unsigned char ch);

void putsLCD(const char* p);

void clrLCD();

void printUIntLCD (uint8_t i);

#endif
